@@ -3,7 +3,8 @@ package ovn
 import (
 	"fmt"
 
-	util "github.com/openvswitch/ovn-kubernetes/go-controller/pkg/util"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
+	util "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 	"github.com/sirupsen/logrus"
 	kapi "k8s.io/api/core/v1"
 )
@@ -13,25 +14,7 @@ type lbEndpoints struct {
 	Port int32
 }
 
-// AddEndpoints adds endpoints and creates corresponding resources in OVN
-func (ovn *Controller) AddEndpoints(ep *kapi.Endpoints) error {
-	// get service
-	// TODO: cache the service
-	svc, err := ovn.kube.GetService(ep.Namespace, ep.Name)
-	if err != nil {
-		// This is not necessarily an error. For e.g when there are endpoints
-		// without a corresponding service.
-		logrus.Debugf("no service found for endpoint %s in namespace %s",
-			ep.Name, ep.Namespace)
-		return nil
-	}
-	if !util.IsServiceIPSet(svc) {
-		logrus.Debugf("Skipping service %s due to clusterIP = %q",
-			svc.Name, svc.Spec.ClusterIP)
-		return nil
-	}
-	tcpPortMap := make(map[string]lbEndpoints)
-	udpPortMap := make(map[string]lbEndpoints)
+func (ovn *Controller) getLbEndpoints(ep *kapi.Endpoints, tcpPortMap, udpPortMap map[string]lbEndpoints) {
 	for _, s := range ep.Subsets {
 		for _, ip := range s.Addresses {
 			for _, port := range s.Ports {
@@ -52,15 +35,35 @@ func (ovn *Controller) AddEndpoints(ep *kapi.Endpoints) error {
 			}
 		}
 	}
-
 	logrus.Debugf("Tcp table: %v\nUdp table: %v", tcpPortMap, udpPortMap)
+}
 
+// AddEndpoints adds endpoints and creates corresponding resources in OVN
+func (ovn *Controller) AddEndpoints(ep *kapi.Endpoints) error {
+	// get service
+	// TODO: cache the service
+	svc, err := ovn.kube.GetService(ep.Namespace, ep.Name)
+	if err != nil {
+		// This is not necessarily an error. For e.g when there are endpoints
+		// without a corresponding service.
+		logrus.Debugf("no service found for endpoint %s in namespace %s",
+			ep.Name, ep.Namespace)
+		return nil
+	}
+	if !util.IsClusterIPSet(svc) {
+		logrus.Debugf("Skipping service %s due to clusterIP = %q",
+			svc.Name, svc.Spec.ClusterIP)
+		return nil
+	}
+	tcpPortMap := make(map[string]lbEndpoints)
+	udpPortMap := make(map[string]lbEndpoints)
+	ovn.getLbEndpoints(ep, tcpPortMap, udpPortMap)
 	for svcPortName, lbEps := range tcpPortMap {
 		ips := lbEps.IPs
 		targetPort := lbEps.Port
 		for _, svcPort := range svc.Spec.Ports {
 			if svcPort.Protocol == kapi.ProtocolTCP && svcPort.Name == svcPortName {
-				if svc.Spec.Type == kapi.ServiceTypeNodePort && ovn.nodePortEnable {
+				if util.ServiceTypeHasNodePort(svc) && config.Gateway.NodeportEnable {
 					logrus.Debugf("Creating Gateways IP for NodePort: %d, %v", svcPort.NodePort, ips)
 					err = ovn.createGatewaysVIP(string(svcPort.Protocol), svcPort.NodePort, targetPort, ips)
 					if err != nil {
@@ -68,7 +71,7 @@ func (ovn *Controller) AddEndpoints(ep *kapi.Endpoints) error {
 						continue
 					}
 				}
-				if svc.Spec.Type == kapi.ServiceTypeClusterIP || svc.Spec.Type == kapi.ServiceTypeNodePort {
+				if util.ServiceTypeHasClusterIP(svc) {
 					var loadBalancer string
 					loadBalancer, err = ovn.getLoadBalancer(svcPort.Protocol)
 					if err != nil {
@@ -92,14 +95,14 @@ func (ovn *Controller) AddEndpoints(ep *kapi.Endpoints) error {
 		targetPort := lbEps.Port
 		for _, svcPort := range svc.Spec.Ports {
 			if svcPort.Protocol == kapi.ProtocolUDP && svcPort.Name == svcPortName {
-				if svc.Spec.Type == kapi.ServiceTypeNodePort && ovn.nodePortEnable {
+				if util.ServiceTypeHasNodePort(svc) && config.Gateway.NodeportEnable {
 					err = ovn.createGatewaysVIP(string(svcPort.Protocol), svcPort.NodePort, targetPort, ips)
 					if err != nil {
 						logrus.Errorf("Error in creating Node Port for svc %s, node port: %d - %v\n", svc.Name, svcPort.NodePort, err)
 						continue
 					}
 				}
-				if svc.Spec.Type == kapi.ServiceTypeNodePort || svc.Spec.Type == kapi.ServiceTypeClusterIP {
+				if util.ServiceTypeHasClusterIP(svc) {
 					var loadBalancer string
 					loadBalancer, err = ovn.getLoadBalancer(svcPort.Protocol)
 					if err != nil {
@@ -121,6 +124,76 @@ func (ovn *Controller) AddEndpoints(ep *kapi.Endpoints) error {
 	return nil
 }
 
+func (ovn *Controller) handleNodePortLB(node *kapi.Node) bool {
+	physicalGateway := "GR_" + node.Name
+	var k8sNSLbTCP, k8sNSLbUDP, physicalIP string
+	if k8sNSLbTCP, _ = ovn.getGatewayLoadBalancer(physicalGateway, TCP); k8sNSLbTCP == "" {
+		logrus.Debugf("TCP load balancer for node %q does not yet exist", node.Name)
+		return false
+	}
+	if k8sNSLbUDP, _ = ovn.getGatewayLoadBalancer(physicalGateway, UDP); k8sNSLbUDP == "" {
+		logrus.Debugf("UDP load balancer for node %q does not yet exist", node.Name)
+		return false
+	}
+	if physicalIP, _ = ovn.getGatewayPhysicalIP(physicalGateway); physicalIP == "" {
+		logrus.Debugf("gateway physical IP for node %q does not yet exist", node.Name)
+		return false
+	}
+	namespaces, err := ovn.kube.GetNamespaces()
+	if err != nil {
+		logrus.Errorf("failed to get k8s namespaces: %v", err)
+		return false
+	}
+	for _, ns := range namespaces.Items {
+		endpoints, err := ovn.kube.GetEndpoints(ns.Name)
+		if err != nil {
+			logrus.Errorf("failed to get k8s endpoints: %v", err)
+			continue
+		}
+		for _, ep := range endpoints.Items {
+			svc, err := ovn.kube.GetService(ep.Namespace, ep.Name)
+			if err != nil {
+				continue
+			}
+			if !util.ServiceTypeHasNodePort(svc) {
+				continue
+			}
+			tcpPortMap := make(map[string]lbEndpoints)
+			udpPortMap := make(map[string]lbEndpoints)
+			ovn.getLbEndpoints(&ep, tcpPortMap, udpPortMap)
+			for svcPortName, lbEps := range tcpPortMap {
+				ips := lbEps.IPs
+				targetPort := lbEps.Port
+				for _, svcPort := range svc.Spec.Ports {
+					if svcPort.Protocol == kapi.ProtocolTCP && svcPort.Name == svcPortName {
+						err = ovn.createLoadBalancerVIP(k8sNSLbTCP,
+							physicalIP, svcPort.NodePort, ips, targetPort)
+						if err != nil {
+							logrus.Errorf("failed to create VIP in load balancer %s - %v", k8sNSLbTCP, err)
+							continue
+						}
+					}
+				}
+			}
+			for svcPortName, lbEps := range udpPortMap {
+				ips := lbEps.IPs
+				targetPort := lbEps.Port
+				for _, svcPort := range svc.Spec.Ports {
+					if svcPort.Protocol == kapi.ProtocolUDP && svcPort.Name == svcPortName {
+						err = ovn.createLoadBalancerVIP(k8sNSLbUDP,
+							physicalIP, svcPort.NodePort, ips, targetPort)
+						if err != nil {
+							logrus.Errorf("failed to create VIP in load balancer %s - %v", k8sNSLbUDP, err)
+							continue
+						}
+					}
+				}
+			}
+		}
+	}
+	return true
+}
+
 func (ovn *Controller) handleExternalIPs(svc *kapi.Service, svcPort kapi.ServicePort, ips []string, targetPort int32) {
 	logrus.Debugf("handling external IPs for svc %v", svc.Name)
 	if len(svc.Spec.ExternalIPs) == 0 {
@@ -149,7 +222,7 @@ func (ovn *Controller) deleteEndpoints(ep *kapi.Endpoints) error {
 			ep.Name, ep.Namespace)
 		return nil
 	}
-	if !util.IsServiceIPSet(svc) {
+	if !util.IsClusterIPSet(svc) {
 		return nil
 	}
 	for _, svcPort := range svc.Spec.Ports {
